#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <cmath>

int H,W,K;
int NUMCELLS;

int IN[210][210][3];
int OUT[210][210][3];
int TEMP[210][210][3];

int MOVES[2][10100][6];
int numMoves[2];
double diffs[2];

double curDiff,bestDiff;

int tempId,bestId;

double DIFF[210][210];

float fill_difference() {
	float res = 0;
	for(int i=0;i<H;i++) 
		for(int j=0;j<W;j++) {
			int r = (OUT[i][j][0]-TEMP[i][j][0]);
			int g = (OUT[i][j][1]-TEMP[i][j][1]);
			int b = (OUT[i][j][2]-TEMP[i][j][2]);
			DIFF[i][j] = sqrt((double)r*r+(double)g*g+(double)b*b);
			res+=DIFF[i][j];
		}
	return res;			
}

void fill_temp() {
	memcpy(TEMP,IN,sizeof(TEMP));
}

float evaluate_pixel(int i, int j, int k, int l) {
	int r = (TEMP[i][j][0]+TEMP[k][l][0])/2;
	int g = (TEMP[i][j][1]+TEMP[k][l][1])/2;
	int b = (TEMP[i][j][2]+TEMP[k][l][2])/2;
	r = (OUT[i][j][0]-r);
	g = (OUT[i][j][1]-g);
	b = (OUT[i][j][2]-b);
	double new_diff = sqrt((double)r*r+(double)g*g+(double)b*b);
	return new_diff-DIFF[i][j];
}
double evaluate_move(int i1,int j1,int i2, int j2, int h, int w) {
	double tot_diff = 0;
	for(int i=0;i<h;i++)
		for(int j=0;j<w;j++) 
			for(int ii=0;ii<h;ii++) 
				for(int jj=0;jj<w;jj++) {
					int r = (TEMP[i1+ii][j1+jj][0]+TEMP[i2+ii][j2+jj][0])/2;
					int g = (TEMP[i1+ii][j1+jj][1]+TEMP[i2+ii][j2+jj][1])/2;
					int b = (TEMP[i1+ii][j1+jj][2]+TEMP[i2+ii][j2+jj][2])/2;

					r = (OUT[i1+ii][j1+jj][0]-r);
					g = (OUT[i1+ii][j1+jj][1]-g);
					b = (OUT[i1+ii][j1+jj][2]-b);
					double diff = sqrt((double)r*r+(double)g*g+(double)b*b);
					tot_diff+=(diff-DIFF[i1+ii][j1+jj]);
				}
	return tot_diff;
}

double make_move(int i1,int j1,int i2, int j2, int h, int w) {
	double tot_diff = 0;
	for(int i=0;i<h;i++)
		for(int j=0;j<w;j++) 
			for(int ii=0;ii<h;ii++) 
				for(int jj=0;jj<w;jj++) {
					TEMP[i1+ii][j1+jj][0] = (TEMP[i1+ii][j1+jj][0]+TEMP[i2+ii][j2+jj][0])/2;
					TEMP[i1+ii][j1+jj][1] = (TEMP[i1+ii][j1+jj][1]+TEMP[i2+ii][j2+jj][1])/2;
					TEMP[i1+ii][j1+jj][2] = (TEMP[i1+ii][j1+jj][2]+TEMP[i2+ii][j2+jj][2])/2;

					int r = (OUT[i1+ii][j1+jj][0]-TEMP[i1+ii][j1+jj][0]);
					int g = (OUT[i1+ii][j1+jj][1]-TEMP[i1+ii][j1+jj][1]);
					int b = (OUT[i1+ii][j1+jj][2]-TEMP[i1+ii][j1+jj][2]);
					double diff = sqrt((double)r*r+(double)g*g+(double)b*b);
					DIFF[i1+ii][j1+jj] = diff;
					tot_diff+=diff;
				}
	return tot_diff;
}

int main() {
	srand(31337);

	FILE* rfile = fopen("transformation.in" , "r");
	fscanf(rfile,"%d%d%d\n",&H,&W,&K);
	for(int i=0;i<H;i++)for(int j=0;j<W;j++)fscanf(rfile,"%d",&IN[i][j][0]);
	for(int i=0;i<H;i++)for(int j=0;j<W;j++)fscanf(rfile,"%d",&IN[i][j][1]);
	for(int i=0;i<H;i++)for(int j=0;j<W;j++)fscanf(rfile,"%d",&IN[i][j][2]);
	for(int i=0;i<H;i++)for(int j=0;j<W;j++)fscanf(rfile,"%d",&OUT[i][j][0]);
	for(int i=0;i<H;i++)for(int j=0;j<W;j++)fscanf(rfile,"%d",&OUT[i][j][1]);
	for(int i=0;i<H;i++)for(int j=0;j<W;j++)fscanf(rfile,"%d",&OUT[i][j][2]);
	fclose(rfile);

	NUMCELLS = H*W;
	int TRIES = 12000000/K;
	bestId = 0;
	tempId = 1;
	// FIRST TRY
	fill_temp();
	diffs[tempId] = fill_difference();
	int k;
	for(k=0;k<K;k++) {
		double bestDiff = 10001000;
		int bestI1,bestJ1,bestI2,bestJ2,bestW,bestH;
		int tries = TRIES;
		while(tries>0) {
			int SW,SH;
			if(tries>150) {
				SH = rand()%8+1;
				SW = rand()%8+1;
			}
			else {
				SH = rand()%2+1;
				SW = rand()%2+1;
			}
			
			int SNI = H/SH;
			int SNJ = W/SW;
			int SCELLS = SNI*SNJ;
			int first = rand()%(SCELLS);
			int second = rand()%(SCELLS-1);
			if(second>=first)second++;
			int i1=(first/SNJ)*SH;
			int j1=(first%SNJ)*SW;
			int i2=(second/SNJ)*SH;
			int j2=(second%SNJ)*SW;
			double delta = evaluate_move(i1,j1,i2,j2,SH,SW);
			if(delta<bestDiff) {
				bestDiff = delta;
				bestI1=i1;
				bestJ1=j1;
				bestI2=i2;
				bestJ2=j2;
				bestW=SW;
				bestH=SH;
			}
			tries-=(SW*SH);
		}
		if(bestDiff>=0) break;
		else {
			diffs[tempId]+=bestDiff;
			make_move(bestI1,bestJ1,bestI2,bestJ2,bestW,bestH);
			MOVES[tempId][k][0]=bestI1;
			MOVES[tempId][k][1]=bestJ1;
			MOVES[tempId][k][2]=bestI2;
			MOVES[tempId][k][3]=bestJ2;
			MOVES[tempId][k][4]=bestH;
			MOVES[tempId][k][5]=bestW;
		}
	}
	numMoves[tempId] = k;

	bestId = (bestId+1)%2;
	tempId = (tempId+1)%2;
	FILE* wfile = fopen("transformation.out" , "w");
	fprintf(wfile,"%d\n",numMoves[bestId]);
	for(int i=0;i<numMoves[bestId];i++)
		fprintf(wfile,"%d %d %d %d %d %d\n",MOVES[bestId][i][0],MOVES[bestId][i][1],MOVES[bestId][i][2],MOVES[bestId][i][3],MOVES[bestId][i][4],MOVES[bestId][i][5]);
	fclose(wfile);
	return 0;
}
